# Copyright (c) Microsoft Corporation.
# Licensed under the MIT license.
"""Strongly-typed return type assertions for every public emit_* method.

Every builder ``emit_*`` method must return the specific ``ResponseStreamEvent``
subtype per spec (e.g. ``emit_added()`` on a message
builder returns ``ResponseOutputItemAddedEvent``, not the base
``ResponseStreamEvent``).  These tests assert the ``isinstance`` contract for
every public emit method on every builder class.

Builder classes covered:
  - ResponseEventStream (lifecycle: queued, created, in_progress, completed, failed, incomplete)
  - OutputItemBuilder (generic: emit_added, emit_done)
  - OutputItemMessageBuilder (emit_added, emit_done)
  - TextContentBuilder (emit_added, emit_delta, emit_text_done, emit_done, emit_annotation_added)
  - RefusalContentBuilder (emit_added, emit_delta, emit_refusal_done, emit_done)
  - OutputItemFunctionCallBuilder (emit_added, emit_arguments_delta, emit_arguments_done, emit_done)
  - OutputItemFunctionCallOutputBuilder (emit_added, emit_done)
  - OutputItemReasoningItemBuilder (emit_added, emit_done)
  - ReasoningSummaryPartBuilder (emit_added, emit_text_delta, emit_text_done, emit_done)
  - OutputItemFileSearchCallBuilder (emit_added, emit_in_progress, emit_searching, emit_completed, emit_done)
  - OutputItemWebSearchCallBuilder (emit_added, emit_in_progress, emit_searching, emit_completed, emit_done)
  - OutputItemCodeInterpreterCallBuilder (emit_added, emit_in_progress, emit_interpreting,
      emit_code_delta, emit_code_done, emit_completed, emit_done)
  - OutputItemImageGenCallBuilder (emit_added, emit_in_progress, emit_generating,
      emit_partial_image, emit_completed, emit_done)
  - OutputItemMcpCallBuilder (emit_added, emit_in_progress, emit_arguments_delta,
      emit_arguments_done, emit_completed, emit_failed, emit_done)
  - OutputItemMcpListToolsBuilder (emit_added, emit_in_progress, emit_completed,
      emit_failed, emit_done)
  - OutputItemCustomToolCallBuilder (emit_added, emit_input_delta, emit_input_done, emit_done)
"""

from __future__ import annotations

from azure.ai.agentserver.responses.models._generated import (
    ResponseCodeInterpreterCallCodeDeltaEvent,
    ResponseCodeInterpreterCallCodeDoneEvent,
    ResponseCodeInterpreterCallCompletedEvent,
    ResponseCodeInterpreterCallInProgressEvent,
    ResponseCodeInterpreterCallInterpretingEvent,
    ResponseCompletedEvent,
    ResponseContentPartAddedEvent,
    ResponseContentPartDoneEvent,
    ResponseCreatedEvent,
    ResponseCustomToolCallInputDeltaEvent,
    ResponseCustomToolCallInputDoneEvent,
    ResponseFailedEvent,
    ResponseFileSearchCallCompletedEvent,
    ResponseFileSearchCallInProgressEvent,
    ResponseFileSearchCallSearchingEvent,
    ResponseFunctionCallArgumentsDeltaEvent,
    ResponseFunctionCallArgumentsDoneEvent,
    ResponseImageGenCallCompletedEvent,
    ResponseImageGenCallGeneratingEvent,
    ResponseImageGenCallInProgressEvent,
    ResponseImageGenCallPartialImageEvent,
    ResponseIncompleteEvent,
    ResponseInProgressEvent,
    ResponseMCPCallArgumentsDeltaEvent,
    ResponseMCPCallArgumentsDoneEvent,
    ResponseMCPCallCompletedEvent,
    ResponseMCPCallFailedEvent,
    ResponseMCPCallInProgressEvent,
    ResponseMCPListToolsCompletedEvent,
    ResponseMCPListToolsFailedEvent,
    ResponseMCPListToolsInProgressEvent,
    ResponseOutputItemAddedEvent,
    ResponseOutputItemDoneEvent,
    ResponseOutputTextAnnotationAddedEvent,
    ResponseQueuedEvent,
    ResponseReasoningSummaryPartAddedEvent,
    ResponseReasoningSummaryPartDoneEvent,
    ResponseReasoningSummaryTextDeltaEvent,
    ResponseReasoningSummaryTextDoneEvent,
    ResponseRefusalDeltaEvent,
    ResponseRefusalDoneEvent,
    ResponseTextDeltaEvent,
    ResponseTextDoneEvent,
    ResponseWebSearchCallCompletedEvent,
    ResponseWebSearchCallInProgressEvent,
    ResponseWebSearchCallSearchingEvent,
    StructuredOutputsOutputItem,
    UrlCitationBody,
)
from azure.ai.agentserver.responses.streaming._event_stream import ResponseEventStream

# ---- helper ----


def _stream() -> ResponseEventStream:
    return ResponseEventStream(response_id="resp_emit_types")


# =====================================================================
# ResponseEventStream lifecycle methods
# =====================================================================


class TestLifecycleReturnTypes:
    """Lifecycle emit methods on ResponseEventStream must return specific subtypes."""

    def test_emit_queued(self) -> None:
        # The state-machine validator requires ``response.created`` as the first
        # event so we cannot call ``emit_queued`` in isolation.  Instead, verify
        # the underlying cast by constructing the model directly – the same
        # ``construct_event_model`` helper that ``_emit_event`` delegates to.
        from azure.ai.agentserver.responses.streaming._internals import construct_event_model

        raw = {
            "type": "response.queued",
            "sequence_number": 0,
            "response": {"id": "r1", "object": "response", "output": []},
        }
        model = construct_event_model(raw)
        assert isinstance(model, ResponseQueuedEvent)

    def test_emit_created(self) -> None:
        s = _stream()
        event = s.emit_created()
        assert isinstance(event, ResponseCreatedEvent), f"Expected ResponseCreatedEvent, got {type(event)}"

    def test_emit_in_progress(self) -> None:
        s = _stream()
        s.emit_created()
        event = s.emit_in_progress()
        assert isinstance(event, ResponseInProgressEvent), f"Expected ResponseInProgressEvent, got {type(event)}"

    def test_emit_completed(self) -> None:
        s = _stream()
        s.emit_created()
        s.emit_in_progress()
        # Need at least one output item before completed
        msg = s.add_output_item_message()
        msg.emit_added()
        tc = msg.add_text_content()
        tc.emit_added()
        tc.emit_delta("x")
        tc.emit_text_done()
        tc.emit_done()
        msg.emit_done()
        event = s.emit_completed()
        assert isinstance(event, ResponseCompletedEvent), f"Expected ResponseCompletedEvent, got {type(event)}"

    def test_emit_failed(self) -> None:
        s = _stream()
        s.emit_created()
        s.emit_in_progress()
        event = s.emit_failed()
        assert isinstance(event, ResponseFailedEvent), f"Expected ResponseFailedEvent, got {type(event)}"

    def test_emit_incomplete(self) -> None:
        s = _stream()
        s.emit_created()
        s.emit_in_progress()
        event = s.emit_incomplete(reason="max_output_tokens")
        assert isinstance(event, ResponseIncompleteEvent), f"Expected ResponseIncompleteEvent, got {type(event)}"


# =====================================================================
# OutputItemBuilder (generic)
# =====================================================================


class TestGenericOutputItemBuilderReturnTypes:
    """Generic OutputItemBuilder.emit_added/emit_done must return typed events."""

    def test_emit_added_returns_output_item_added_event(self) -> None:
        s = _stream()
        s.emit_created()
        builder = s.add_output_item_structured_outputs()
        item = StructuredOutputsOutputItem(id=builder.item_id, output="data")
        event = builder.emit_added(item)
        assert isinstance(event, ResponseOutputItemAddedEvent), (
            f"Expected ResponseOutputItemAddedEvent, got {type(event)}"
        )

    def test_emit_done_returns_output_item_done_event(self) -> None:
        s = _stream()
        s.emit_created()
        builder = s.add_output_item_structured_outputs()
        item = StructuredOutputsOutputItem(id=builder.item_id, output="data")
        builder.emit_added(item)
        event = builder.emit_done(item)
        assert isinstance(event, ResponseOutputItemDoneEvent), (
            f"Expected ResponseOutputItemDoneEvent, got {type(event)}"
        )


# =====================================================================
# OutputItemMessageBuilder
# =====================================================================


class TestMessageBuilderReturnTypes:
    """OutputItemMessageBuilder emit methods must return typed events."""

    def test_emit_added(self) -> None:
        s = _stream()
        s.emit_created()
        msg = s.add_output_item_message()
        event = msg.emit_added()
        assert isinstance(event, ResponseOutputItemAddedEvent)

    def test_emit_done(self) -> None:
        s = _stream()
        s.emit_created()
        msg = s.add_output_item_message()
        msg.emit_added()
        tc = msg.add_text_content()
        tc.emit_added()
        tc.emit_delta("x")
        tc.emit_text_done()
        tc.emit_done()
        event = msg.emit_done()
        assert isinstance(event, ResponseOutputItemDoneEvent)


# =====================================================================
# TextContentBuilder
# =====================================================================


class TestTextContentBuilderReturnTypes:
    """TextContentBuilder emit methods must return typed events."""

    def _setup(self):
        s = _stream()
        s.emit_created()
        msg = s.add_output_item_message()
        msg.emit_added()
        tc = msg.add_text_content()
        return s, msg, tc

    def test_emit_added(self) -> None:
        _, _, tc = self._setup()
        event = tc.emit_added()
        assert isinstance(event, ResponseContentPartAddedEvent), (
            f"Expected ResponseContentPartAddedEvent, got {type(event)}"
        )

    def test_emit_delta(self) -> None:
        _, _, tc = self._setup()
        tc.emit_added()
        event = tc.emit_delta("hello")
        assert isinstance(event, ResponseTextDeltaEvent), f"Expected ResponseTextDeltaEvent, got {type(event)}"

    def test_emit_text_done(self) -> None:
        _, _, tc = self._setup()
        tc.emit_added()
        tc.emit_delta("hello")
        event = tc.emit_text_done()
        assert isinstance(event, ResponseTextDoneEvent), f"Expected ResponseTextDoneEvent, got {type(event)}"

    def test_emit_done(self) -> None:
        _, _, tc = self._setup()
        tc.emit_added()
        tc.emit_delta("hello")
        tc.emit_text_done()
        event = tc.emit_done()
        assert isinstance(event, ResponseContentPartDoneEvent), (
            f"Expected ResponseContentPartDoneEvent, got {type(event)}"
        )

    def test_emit_annotation_added(self) -> None:
        _, _, tc = self._setup()
        tc.emit_added()
        tc.emit_delta("hello")
        tc.emit_text_done()
        annotation = UrlCitationBody(
            url="https://example.com",
            start_index=0,
            end_index=5,
            title="Example",
        )
        event = tc.emit_annotation_added(annotation)
        assert isinstance(event, ResponseOutputTextAnnotationAddedEvent), (
            f"Expected ResponseOutputTextAnnotationAddedEvent, got {type(event)}"
        )


# =====================================================================
# RefusalContentBuilder
# =====================================================================


class TestRefusalContentBuilderReturnTypes:
    """RefusalContentBuilder emit methods must return typed events."""

    def _setup(self):
        s = _stream()
        s.emit_created()
        msg = s.add_output_item_message()
        msg.emit_added()
        rc = msg.add_refusal_content()
        return s, msg, rc

    def test_emit_added(self) -> None:
        _, _, rc = self._setup()
        event = rc.emit_added()
        assert isinstance(event, ResponseContentPartAddedEvent)

    def test_emit_delta(self) -> None:
        _, _, rc = self._setup()
        rc.emit_added()
        event = rc.emit_delta("I cannot do that")
        assert isinstance(event, ResponseRefusalDeltaEvent), f"Expected ResponseRefusalDeltaEvent, got {type(event)}"

    def test_emit_refusal_done(self) -> None:
        _, _, rc = self._setup()
        rc.emit_added()
        rc.emit_delta("I cannot do that")
        event = rc.emit_refusal_done("I cannot do that")
        assert isinstance(event, ResponseRefusalDoneEvent), f"Expected ResponseRefusalDoneEvent, got {type(event)}"

    def test_emit_done(self) -> None:
        _, _, rc = self._setup()
        rc.emit_added()
        rc.emit_delta("I cannot do that")
        rc.emit_refusal_done("I cannot do that")
        event = rc.emit_done()
        assert isinstance(event, ResponseContentPartDoneEvent)


# =====================================================================
# OutputItemFunctionCallBuilder
# =====================================================================


class TestFunctionCallBuilderReturnTypes:
    """OutputItemFunctionCallBuilder emit methods must return typed events."""

    def test_emit_added(self) -> None:
        s = _stream()
        s.emit_created()
        fc = s.add_output_item_function_call("fn", "call_1")
        event = fc.emit_added()
        assert isinstance(event, ResponseOutputItemAddedEvent)

    def test_emit_arguments_delta(self) -> None:
        s = _stream()
        s.emit_created()
        fc = s.add_output_item_function_call("fn", "call_1")
        fc.emit_added()
        event = fc.emit_arguments_delta('{"k":')
        assert isinstance(event, ResponseFunctionCallArgumentsDeltaEvent), (
            f"Expected ResponseFunctionCallArgumentsDeltaEvent, got {type(event)}"
        )

    def test_emit_arguments_done(self) -> None:
        s = _stream()
        s.emit_created()
        fc = s.add_output_item_function_call("fn", "call_1")
        fc.emit_added()
        event = fc.emit_arguments_done('{"k":"v"}')
        assert isinstance(event, ResponseFunctionCallArgumentsDoneEvent), (
            f"Expected ResponseFunctionCallArgumentsDoneEvent, got {type(event)}"
        )

    def test_emit_done(self) -> None:
        s = _stream()
        s.emit_created()
        fc = s.add_output_item_function_call("fn", "call_1")
        fc.emit_added()
        fc.emit_arguments_done("{}")
        event = fc.emit_done()
        assert isinstance(event, ResponseOutputItemDoneEvent)


# =====================================================================
# OutputItemFunctionCallOutputBuilder
# =====================================================================


class TestFunctionCallOutputBuilderReturnTypes:
    """OutputItemFunctionCallOutputBuilder emit methods must return typed events."""

    def test_emit_added(self) -> None:
        s = _stream()
        s.emit_created()
        fco = s.add_output_item_function_call_output("call_1")
        event = fco.emit_added("result")
        assert isinstance(event, ResponseOutputItemAddedEvent)

    def test_emit_done(self) -> None:
        s = _stream()
        s.emit_created()
        fco = s.add_output_item_function_call_output("call_1")
        fco.emit_added("result")
        event = fco.emit_done("result")
        assert isinstance(event, ResponseOutputItemDoneEvent)


# =====================================================================
# OutputItemReasoningItemBuilder
# =====================================================================


class TestReasoningItemBuilderReturnTypes:
    """OutputItemReasoningItemBuilder emit methods must return typed events."""

    def test_emit_added(self) -> None:
        s = _stream()
        s.emit_created()
        r = s.add_output_item_reasoning_item()
        event = r.emit_added()
        assert isinstance(event, ResponseOutputItemAddedEvent)

    def test_emit_done(self) -> None:
        s = _stream()
        s.emit_created()
        r = s.add_output_item_reasoning_item()
        r.emit_added()
        sp = r.add_summary_part()
        sp.emit_added()
        sp.emit_text_done("thinking")
        sp.emit_done()
        event = r.emit_done()
        assert isinstance(event, ResponseOutputItemDoneEvent)


# =====================================================================
# ReasoningSummaryPartBuilder
# =====================================================================


class TestReasoningSummaryPartBuilderReturnTypes:
    """ReasoningSummaryPartBuilder emit methods must return typed events."""

    def _setup(self):
        s = _stream()
        s.emit_created()
        r = s.add_output_item_reasoning_item()
        r.emit_added()
        sp = r.add_summary_part()
        return s, r, sp

    def test_emit_added(self) -> None:
        _, _, sp = self._setup()
        event = sp.emit_added()
        assert isinstance(event, ResponseReasoningSummaryPartAddedEvent), (
            f"Expected ResponseReasoningSummaryPartAddedEvent, got {type(event)}"
        )

    def test_emit_text_delta(self) -> None:
        _, _, sp = self._setup()
        sp.emit_added()
        event = sp.emit_text_delta("thinking")
        assert isinstance(event, ResponseReasoningSummaryTextDeltaEvent), (
            f"Expected ResponseReasoningSummaryTextDeltaEvent, got {type(event)}"
        )

    def test_emit_text_done(self) -> None:
        _, _, sp = self._setup()
        sp.emit_added()
        event = sp.emit_text_done("thinking")
        assert isinstance(event, ResponseReasoningSummaryTextDoneEvent), (
            f"Expected ResponseReasoningSummaryTextDoneEvent, got {type(event)}"
        )

    def test_emit_done(self) -> None:
        _, _, sp = self._setup()
        sp.emit_added()
        sp.emit_text_done("thinking")
        event = sp.emit_done()
        assert isinstance(event, ResponseReasoningSummaryPartDoneEvent), (
            f"Expected ResponseReasoningSummaryPartDoneEvent, got {type(event)}"
        )


# =====================================================================
# OutputItemFileSearchCallBuilder
# =====================================================================


class TestFileSearchCallBuilderReturnTypes:
    """OutputItemFileSearchCallBuilder emit methods must return typed events."""

    def test_emit_added(self) -> None:
        s = _stream()
        s.emit_created()
        fs = s.add_output_item_file_search_call()
        event = fs.emit_added()
        assert isinstance(event, ResponseOutputItemAddedEvent)

    def test_emit_in_progress(self) -> None:
        s = _stream()
        s.emit_created()
        fs = s.add_output_item_file_search_call()
        fs.emit_added()
        event = fs.emit_in_progress()
        assert isinstance(event, ResponseFileSearchCallInProgressEvent), (
            f"Expected ResponseFileSearchCallInProgressEvent, got {type(event)}"
        )

    def test_emit_searching(self) -> None:
        s = _stream()
        s.emit_created()
        fs = s.add_output_item_file_search_call()
        fs.emit_added()
        fs.emit_in_progress()
        event = fs.emit_searching()
        assert isinstance(event, ResponseFileSearchCallSearchingEvent), (
            f"Expected ResponseFileSearchCallSearchingEvent, got {type(event)}"
        )

    def test_emit_completed(self) -> None:
        s = _stream()
        s.emit_created()
        fs = s.add_output_item_file_search_call()
        fs.emit_added()
        event = fs.emit_completed()
        assert isinstance(event, ResponseFileSearchCallCompletedEvent), (
            f"Expected ResponseFileSearchCallCompletedEvent, got {type(event)}"
        )

    def test_emit_done(self) -> None:
        s = _stream()
        s.emit_created()
        fs = s.add_output_item_file_search_call()
        fs.emit_added()
        event = fs.emit_done()
        assert isinstance(event, ResponseOutputItemDoneEvent)


# =====================================================================
# OutputItemWebSearchCallBuilder
# =====================================================================


class TestWebSearchCallBuilderReturnTypes:
    """OutputItemWebSearchCallBuilder emit methods must return typed events."""

    def test_emit_added(self) -> None:
        s = _stream()
        s.emit_created()
        ws = s.add_output_item_web_search_call()
        event = ws.emit_added()
        assert isinstance(event, ResponseOutputItemAddedEvent)

    def test_emit_in_progress(self) -> None:
        s = _stream()
        s.emit_created()
        ws = s.add_output_item_web_search_call()
        ws.emit_added()
        event = ws.emit_in_progress()
        assert isinstance(event, ResponseWebSearchCallInProgressEvent), (
            f"Expected ResponseWebSearchCallInProgressEvent, got {type(event)}"
        )

    def test_emit_searching(self) -> None:
        s = _stream()
        s.emit_created()
        ws = s.add_output_item_web_search_call()
        ws.emit_added()
        event = ws.emit_searching()
        assert isinstance(event, ResponseWebSearchCallSearchingEvent), (
            f"Expected ResponseWebSearchCallSearchingEvent, got {type(event)}"
        )

    def test_emit_completed(self) -> None:
        s = _stream()
        s.emit_created()
        ws = s.add_output_item_web_search_call()
        ws.emit_added()
        event = ws.emit_completed()
        assert isinstance(event, ResponseWebSearchCallCompletedEvent), (
            f"Expected ResponseWebSearchCallCompletedEvent, got {type(event)}"
        )

    def test_emit_done(self) -> None:
        s = _stream()
        s.emit_created()
        ws = s.add_output_item_web_search_call()
        ws.emit_added()
        event = ws.emit_done()
        assert isinstance(event, ResponseOutputItemDoneEvent)


# =====================================================================
# OutputItemCodeInterpreterCallBuilder
# =====================================================================


class TestCodeInterpreterCallBuilderReturnTypes:
    """OutputItemCodeInterpreterCallBuilder emit methods must return typed events."""

    def test_emit_added(self) -> None:
        s = _stream()
        s.emit_created()
        ci = s.add_output_item_code_interpreter_call()
        event = ci.emit_added()
        assert isinstance(event, ResponseOutputItemAddedEvent)

    def test_emit_in_progress(self) -> None:
        s = _stream()
        s.emit_created()
        ci = s.add_output_item_code_interpreter_call()
        ci.emit_added()
        event = ci.emit_in_progress()
        assert isinstance(event, ResponseCodeInterpreterCallInProgressEvent), (
            f"Expected ResponseCodeInterpreterCallInProgressEvent, got {type(event)}"
        )

    def test_emit_interpreting(self) -> None:
        s = _stream()
        s.emit_created()
        ci = s.add_output_item_code_interpreter_call()
        ci.emit_added()
        event = ci.emit_interpreting()
        assert isinstance(event, ResponseCodeInterpreterCallInterpretingEvent), (
            f"Expected ResponseCodeInterpreterCallInterpretingEvent, got {type(event)}"
        )

    def test_emit_code_delta(self) -> None:
        s = _stream()
        s.emit_created()
        ci = s.add_output_item_code_interpreter_call()
        ci.emit_added()
        event = ci.emit_code_delta("print('hello')")
        assert isinstance(event, ResponseCodeInterpreterCallCodeDeltaEvent), (
            f"Expected ResponseCodeInterpreterCallCodeDeltaEvent, got {type(event)}"
        )

    def test_emit_code_done(self) -> None:
        s = _stream()
        s.emit_created()
        ci = s.add_output_item_code_interpreter_call()
        ci.emit_added()
        event = ci.emit_code_done("print('hello')")
        assert isinstance(event, ResponseCodeInterpreterCallCodeDoneEvent), (
            f"Expected ResponseCodeInterpreterCallCodeDoneEvent, got {type(event)}"
        )

    def test_emit_completed(self) -> None:
        s = _stream()
        s.emit_created()
        ci = s.add_output_item_code_interpreter_call()
        ci.emit_added()
        event = ci.emit_completed()
        assert isinstance(event, ResponseCodeInterpreterCallCompletedEvent), (
            f"Expected ResponseCodeInterpreterCallCompletedEvent, got {type(event)}"
        )

    def test_emit_done(self) -> None:
        s = _stream()
        s.emit_created()
        ci = s.add_output_item_code_interpreter_call()
        ci.emit_added()
        event = ci.emit_done()
        assert isinstance(event, ResponseOutputItemDoneEvent)


# =====================================================================
# OutputItemImageGenCallBuilder
# =====================================================================


class TestImageGenCallBuilderReturnTypes:
    """OutputItemImageGenCallBuilder emit methods must return typed events."""

    def test_emit_added(self) -> None:
        s = _stream()
        s.emit_created()
        ig = s.add_output_item_image_gen_call()
        event = ig.emit_added()
        assert isinstance(event, ResponseOutputItemAddedEvent)

    def test_emit_in_progress(self) -> None:
        s = _stream()
        s.emit_created()
        ig = s.add_output_item_image_gen_call()
        ig.emit_added()
        event = ig.emit_in_progress()
        assert isinstance(event, ResponseImageGenCallInProgressEvent), (
            f"Expected ResponseImageGenCallInProgressEvent, got {type(event)}"
        )

    def test_emit_generating(self) -> None:
        s = _stream()
        s.emit_created()
        ig = s.add_output_item_image_gen_call()
        ig.emit_added()
        event = ig.emit_generating()
        assert isinstance(event, ResponseImageGenCallGeneratingEvent), (
            f"Expected ResponseImageGenCallGeneratingEvent, got {type(event)}"
        )

    def test_emit_partial_image(self) -> None:
        s = _stream()
        s.emit_created()
        ig = s.add_output_item_image_gen_call()
        ig.emit_added()
        event = ig.emit_partial_image("base64data")
        assert isinstance(event, ResponseImageGenCallPartialImageEvent), (
            f"Expected ResponseImageGenCallPartialImageEvent, got {type(event)}"
        )

    def test_emit_completed(self) -> None:
        s = _stream()
        s.emit_created()
        ig = s.add_output_item_image_gen_call()
        ig.emit_added()
        event = ig.emit_completed()
        assert isinstance(event, ResponseImageGenCallCompletedEvent), (
            f"Expected ResponseImageGenCallCompletedEvent, got {type(event)}"
        )

    def test_emit_done(self) -> None:
        s = _stream()
        s.emit_created()
        ig = s.add_output_item_image_gen_call()
        ig.emit_added()
        event = ig.emit_done("final_b64_image")
        assert isinstance(event, ResponseOutputItemDoneEvent)


# =====================================================================
# OutputItemMcpCallBuilder
# =====================================================================


class TestMcpCallBuilderReturnTypes:
    """OutputItemMcpCallBuilder emit methods must return typed events."""

    def test_emit_added(self) -> None:
        s = _stream()
        s.emit_created()
        mcp = s.add_output_item_mcp_call("server", "tool")
        event = mcp.emit_added()
        assert isinstance(event, ResponseOutputItemAddedEvent)

    def test_emit_in_progress(self) -> None:
        s = _stream()
        s.emit_created()
        mcp = s.add_output_item_mcp_call("server", "tool")
        mcp.emit_added()
        event = mcp.emit_in_progress()
        assert isinstance(event, ResponseMCPCallInProgressEvent), (
            f"Expected ResponseMCPCallInProgressEvent, got {type(event)}"
        )

    def test_emit_arguments_delta(self) -> None:
        s = _stream()
        s.emit_created()
        mcp = s.add_output_item_mcp_call("server", "tool")
        mcp.emit_added()
        event = mcp.emit_arguments_delta('{"key":')
        assert isinstance(event, ResponseMCPCallArgumentsDeltaEvent), (
            f"Expected ResponseMCPCallArgumentsDeltaEvent, got {type(event)}"
        )

    def test_emit_arguments_done(self) -> None:
        s = _stream()
        s.emit_created()
        mcp = s.add_output_item_mcp_call("server", "tool")
        mcp.emit_added()
        event = mcp.emit_arguments_done('{"key":"val"}')
        assert isinstance(event, ResponseMCPCallArgumentsDoneEvent), (
            f"Expected ResponseMCPCallArgumentsDoneEvent, got {type(event)}"
        )

    def test_emit_completed(self) -> None:
        s = _stream()
        s.emit_created()
        mcp = s.add_output_item_mcp_call("server", "tool")
        mcp.emit_added()
        event = mcp.emit_completed()
        assert isinstance(event, ResponseMCPCallCompletedEvent), (
            f"Expected ResponseMCPCallCompletedEvent, got {type(event)}"
        )

    def test_emit_failed(self) -> None:
        s = _stream()
        s.emit_created()
        mcp = s.add_output_item_mcp_call("server", "tool")
        mcp.emit_added()
        event = mcp.emit_failed()
        assert isinstance(event, ResponseMCPCallFailedEvent), f"Expected ResponseMCPCallFailedEvent, got {type(event)}"

    def test_emit_done(self) -> None:
        s = _stream()
        s.emit_created()
        mcp = s.add_output_item_mcp_call("server", "tool")
        mcp.emit_added()
        mcp.emit_completed()
        event = mcp.emit_done()
        assert isinstance(event, ResponseOutputItemDoneEvent)

    def test_emit_done_with_output_and_error(self) -> None:
        s = _stream()
        s.emit_created()
        mcp = s.add_output_item_mcp_call("server", "tool", item_id="mcp_test")
        mcp.emit_added()
        mcp.emit_failed()
        event = mcp.emit_done(output="ok", error={"reason": "failed"})
        assert isinstance(event, ResponseOutputItemDoneEvent)


# =====================================================================
# OutputItemMcpListToolsBuilder
# =====================================================================


class TestMcpListToolsBuilderReturnTypes:
    """OutputItemMcpListToolsBuilder emit methods must return typed events."""

    def test_emit_added(self) -> None:
        s = _stream()
        s.emit_created()
        mlt = s.add_output_item_mcp_list_tools("server")
        event = mlt.emit_added()
        assert isinstance(event, ResponseOutputItemAddedEvent)

    def test_emit_in_progress(self) -> None:
        s = _stream()
        s.emit_created()
        mlt = s.add_output_item_mcp_list_tools("server")
        mlt.emit_added()
        event = mlt.emit_in_progress()
        assert isinstance(event, ResponseMCPListToolsInProgressEvent), (
            f"Expected ResponseMCPListToolsInProgressEvent, got {type(event)}"
        )

    def test_emit_completed(self) -> None:
        s = _stream()
        s.emit_created()
        mlt = s.add_output_item_mcp_list_tools("server")
        mlt.emit_added()
        event = mlt.emit_completed()
        assert isinstance(event, ResponseMCPListToolsCompletedEvent), (
            f"Expected ResponseMCPListToolsCompletedEvent, got {type(event)}"
        )

    def test_emit_failed(self) -> None:
        s = _stream()
        s.emit_created()
        mlt = s.add_output_item_mcp_list_tools("server")
        mlt.emit_added()
        event = mlt.emit_failed()
        assert isinstance(event, ResponseMCPListToolsFailedEvent), (
            f"Expected ResponseMCPListToolsFailedEvent, got {type(event)}"
        )

    def test_emit_done(self) -> None:
        s = _stream()
        s.emit_created()
        mlt = s.add_output_item_mcp_list_tools("server")
        mlt.emit_added()
        event = mlt.emit_done()
        assert isinstance(event, ResponseOutputItemDoneEvent)


# =====================================================================
# OutputItemCustomToolCallBuilder
# =====================================================================


class TestCustomToolCallBuilderReturnTypes:
    """OutputItemCustomToolCallBuilder emit methods must return typed events."""

    def test_emit_added(self) -> None:
        s = _stream()
        s.emit_created()
        ct = s.add_output_item_custom_tool_call("call_1", "my_tool")
        event = ct.emit_added()
        assert isinstance(event, ResponseOutputItemAddedEvent)

    def test_emit_input_delta(self) -> None:
        s = _stream()
        s.emit_created()
        ct = s.add_output_item_custom_tool_call("call_1", "my_tool")
        ct.emit_added()
        event = ct.emit_input_delta('{"key":')
        assert isinstance(event, ResponseCustomToolCallInputDeltaEvent), (
            f"Expected ResponseCustomToolCallInputDeltaEvent, got {type(event)}"
        )

    def test_emit_input_done(self) -> None:
        s = _stream()
        s.emit_created()
        ct = s.add_output_item_custom_tool_call("call_1", "my_tool")
        ct.emit_added()
        event = ct.emit_input_done('{"key":"val"}')
        assert isinstance(event, ResponseCustomToolCallInputDoneEvent), (
            f"Expected ResponseCustomToolCallInputDoneEvent, got {type(event)}"
        )

    def test_emit_done(self) -> None:
        s = _stream()
        s.emit_created()
        ct = s.add_output_item_custom_tool_call("call_1", "my_tool")
        ct.emit_added()
        ct.emit_input_done("{}")
        event = ct.emit_done()
        assert isinstance(event, ResponseOutputItemDoneEvent)



# coding: utf-8
# -------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See LICENSE.txt in the project root for
# license information.
# -------------------------------------------------------------------------
from azure.core import PipelineClient
from azure.core.pipeline import Pipeline
from azure.core.pipeline.transport import (
    HttpRequest,
    RequestsTransport,
)
from azure.core.exceptions import IncompleteReadError
import pytest


def test_sync_transport_short_read_download_stream(port):
    url = "http://localhost:{}/errors/short-data".format(port)
    client = PipelineClient(url)
    request = HttpRequest("GET", url)
    with pytest.raises(IncompleteReadError):
        pipeline_response = client._pipeline.run(request, stream=True)
        response = pipeline_response.http_response
        data = response.stream_download(client._pipeline)
        content = b""
        for d in data:
            content += d



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
# pylint: disable=wrong-import-position

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from ._patch import *  # pylint: disable=unused-wildcard-import


from ._models import (  # type: ignore
    AFDDomain,
    AFDDomainHttpsCustomizedCipherSuiteSet,
    AFDDomainHttpsParameters,
    AFDDomainProperties,
    AFDDomainUpdateParameters,
    AFDDomainUpdatePropertiesParameters,
    AFDEndpoint,
    AFDEndpointProperties,
    AFDEndpointPropertiesUpdateParameters,
    AFDEndpointUpdateParameters,
    AFDOrigin,
    AFDOriginGroup,
    AFDOriginGroupProperties,
    AFDOriginGroupUpdateParameters,
    AFDOriginGroupUpdatePropertiesParameters,
    AFDOriginProperties,
    AFDOriginUpdateParameters,
    AFDOriginUpdatePropertiesParameters,
    AFDStateProperties,
    ActivatedResourceReference,
    AfdPurgeParameters,
    AfdRouteCacheConfiguration,
    AzureFirstPartyManagedCertificateParameters,
    BatchRuleProperties,
    CacheConfiguration,
    CacheExpirationActionParameters,
    CacheKeyQueryStringActionParameters,
    CanMigrateParameters,
    CanMigrateProperties,
    CanMigrateResult,
    CdnCertificateSourceParameters,
    CdnEndpoint,
    CdnManagedHttpsParameters,
    CdnMigrationToAfdParameters,
    CdnWebApplicationFirewallPolicy,
    CdnWebApplicationFirewallPolicyPatchParameters,
    CdnWebApplicationFirewallPolicyProperties,
    CertificateSourceParameters,
    CheckEndpointNameAvailabilityInput,
    CheckEndpointNameAvailabilityOutput,
    CheckHostNameAvailabilityInput,
    CheckNameAvailabilityInput,
    CheckNameAvailabilityOutput,
    CidrIpAddress,
    ClientPortMatchConditionParameters,
    Components18OrqelSchemasWafmetricsresponsePropertiesSeriesItemsPropertiesDataItems,
    Components1Gs0LlpSchemasMetricsresponsePropertiesSeriesItemsPropertiesDataItems,
    ComponentsKpo1PjSchemasWafrankingsresponsePropertiesDataItemsPropertiesMetricsItems,
    CompressionSettings,
    ContinentsResponse,
    ContinentsResponseContinentsItem,
    ContinentsResponseCountryOrRegionsItem,
    CookiesMatchConditionParameters,
    CustomDomain,
    CustomDomainHttpsParameters,
    CustomDomainParameters,
    CustomDomainProperties,
    CustomDomainPropertiesParameters,
    CustomRule,
    CustomRuleList,
    CustomerCertificateParameters,
    DeepCreatedCustomDomain,
    DeepCreatedCustomDomainProperties,
    DeepCreatedOrigin,
    DeepCreatedOriginGroup,
    DeepCreatedOriginGroupProperties,
    DeepCreatedOriginProperties,
    DeliveryRule,
    DeliveryRuleAction,
    DeliveryRuleActionParameters,
    DeliveryRuleCacheExpirationAction,
    DeliveryRuleCacheKeyQueryStringAction,
    DeliveryRuleClientPortCondition,
    DeliveryRuleCondition,
    DeliveryRuleConditionParameters,
    DeliveryRuleCookiesCondition,
    DeliveryRuleHostNameCondition,
    DeliveryRuleHttpVersionCondition,
    DeliveryRuleIsDeviceCondition,
    DeliveryRulePostArgsCondition,
    DeliveryRuleQueryStringCondition,
    DeliveryRuleRemoteAddressCondition,
    DeliveryRuleRequestBodyCondition,
    DeliveryRuleRequestHeaderAction,
    DeliveryRuleRequestHeaderCondition,
    DeliveryRuleRequestMethodCondition,
    DeliveryRuleRequestSchemeCondition,
    DeliveryRuleRequestUriCondition,
    DeliveryRuleResponseHeaderAction,
    DeliveryRuleRouteConfigurationOverrideAction,
    DeliveryRuleServerPortCondition,
    DeliveryRuleSocketAddrCondition,
    DeliveryRuleSslProtocolCondition,
    DeliveryRuleUrlFileExtensionCondition,
    DeliveryRuleUrlFileNameCondition,
    DeliveryRuleUrlPathCondition,
    DimensionProperties,
    DomainValidationProperties,
    EdgeNode,
    EdgeNodeProperties,
    Endpoint,
    EndpointProperties,
    EndpointPropertiesUpdateParameters,
    EndpointPropertiesUpdateParametersDeliveryPolicy,
    EndpointPropertiesUpdateParametersWebApplicationFirewallPolicyLink,
    EndpointUpdateParameters,
    ErrorAdditionalInfo,
    ErrorDetail,
    ErrorResponse,
    GeoFilter,
    HeaderActionParameters,
    HealthProbeParameters,
    HostNameMatchConditionParameters,
    HttpErrorRangeParameters,
    HttpVersionMatchConditionParameters,
    IpAddressGroup,
    IsDeviceMatchConditionParameters,
    KeyVaultCertificateSourceParameters,
    KeyVaultSigningKeyParameters,
    LoadBalancingSettingsParameters,
    LoadParameters,
    LogSpecification,
    ManagedCertificateParameters,
    ManagedRuleDefinition,
    ManagedRuleGroupDefinition,
    ManagedRuleGroupOverride,
    ManagedRuleOverride,
    ManagedRuleSet,
    ManagedRuleSetDefinition,
    ManagedRuleSetDefinitionProperties,
    ManagedRuleSetList,
    ManagedServiceIdentity,
    MatchCondition,
    MetricAvailability,
    MetricSpecification,
    MetricsResponse,
    MetricsResponseSeriesItem,
    MetricsResponseSeriesPropertiesItemsItem,
    MigrateResult,
    MigrateResultProperties,
    MigrationEndpointMapping,
    MigrationErrorType,
    MigrationParameters,
    MigrationWebApplicationFirewallMapping,
    Operation,
    OperationDisplay,
    OperationProperties,
    Origin,
    OriginAuthenticationProperties,
    OriginGroup,
    OriginGroupOverride,
    OriginGroupOverrideAction,
    OriginGroupOverrideActionParameters,
    OriginGroupProperties,
    OriginGroupUpdateParameters,
    OriginGroupUpdatePropertiesParameters,
    OriginProperties,
    OriginUpdateParameters,
    OriginUpdatePropertiesParameters,
    PolicySettings,
    PostArgsMatchConditionParameters,
    Profile,
    ProfileChangeSkuWafMapping,
    ProfileLogScrubbing,
    ProfileProperties,
    ProfilePropertiesUpdateParameters,
    ProfileScrubbingRules,
    ProfileUpdateParameters,
    ProfileUpgradeParameters,
    ProxyResource,
    PurgeParameters,
    QueryStringMatchConditionParameters,
    RankingsResponse,
    RankingsResponseTablesItem,
    RankingsResponseTablesPropertiesItemsItem,
    RankingsResponseTablesPropertiesItemsMetricsItem,
    RateLimitRule,
    RateLimitRuleList,
    RemoteAddressMatchConditionParameters,
    RequestBodyMatchConditionParameters,
    RequestHeaderMatchConditionParameters,
    RequestMethodMatchConditionParameters,
    RequestSchemeMatchConditionParameters,
    RequestUriMatchConditionParameters,
    Resource,
    ResourceReference,
    ResourceUsage,
    ResourcesResponse,
    ResourcesResponseCustomDomainsItem,
    ResourcesResponseEndpointsItem,
    ResourcesResponseEndpointsPropertiesItemsItem,
    ResponseBasedOriginErrorDetectionParameters,
    Route,
    RouteConfigurationOverrideActionParameters,
    RouteProperties,
    RouteUpdateParameters,
    RouteUpdatePropertiesParameters,
    Rule,
    RuleProperties,
    RuleSet,
    RuleSetProperties,
    RuleUpdateParameters,
    RuleUpdatePropertiesParameters,
    Secret,
    SecretParameters,
    SecretProperties,
    SecurityPolicy,
    SecurityPolicyProperties,
    SecurityPolicyPropertiesParameters,
    SecurityPolicyUpdateParameters,
    SecurityPolicyUpdateProperties,
    SecurityPolicyWebApplicationFirewallAssociation,
    SecurityPolicyWebApplicationFirewallParameters,
    ServerPortMatchConditionParameters,
    ServiceSpecification,
    SharedPrivateLinkResourceProperties,
    Sku,
    SocketAddrMatchConditionParameters,
    SslProtocolMatchConditionParameters,
    SsoUri,
    SupportedOptimizationTypesListResult,
    SystemData,
    TrackedResource,
    UrlFileExtensionMatchConditionParameters,
    UrlFileNameMatchConditionParameters,
    UrlPathMatchConditionParameters,
    UrlRedirectAction,
    UrlRedirectActionParameters,
    UrlRewriteAction,
    UrlRewriteActionParameters,
    UrlSigningAction,
    UrlSigningActionParameters,
    UrlSigningKey,
    UrlSigningKeyParameters,
    UrlSigningParamIdentifier,
    Usage,
    UsageName,
    UserAssignedIdentity,
    UserManagedHttpsParameters,
    ValidateCustomDomainInput,
    ValidateCustomDomainOutput,
    ValidateProbeInput,
    ValidateProbeOutput,
    ValidateSecretInput,
    ValidateSecretOutput,
    WafMetricsResponse,
    WafMetricsResponseSeriesItem,
    WafMetricsResponseSeriesPropertiesItemsItem,
    WafRankingsResponse,
    WafRankingsResponseDataItem,
)

from ._enums import (  # type: ignore
    AFDEndpointProtocols,
    ActionType,
    AfdCertificateType,
    AfdCipherSuiteSetType,
    AfdCustomizedCipherSuiteForTls12,
    AfdCustomizedCipherSuiteForTls13,
    AfdMinimumTlsVersion,
    AfdProvisioningState,
    AfdQueryStringCachingBehavior,
    Algorithm,
    AutoGeneratedDomainNameLabelScope,
    CacheBehavior,
    CacheType,
    CanMigrateDefaultSku,
    CertificateSource,
    CertificateSourceParametersType,
    CertificateType,
    ClientPortOperator,
    CookiesOperator,
    CreatedByType,
    CustomDomainResourceState,
    CustomHttpsProvisioningState,
    CustomHttpsProvisioningSubstate,
    CustomRuleEnabledState,
    DeleteRule,
    DeliveryRuleActionEnum,
    DeliveryRuleActionParametersType,
    DeliveryRuleConditionParametersType,
    DeploymentStatus,
    DestinationProtocol,
    DomainValidationState,
    EnabledState,
    EndpointProvisioningState,
    EndpointResourceState,
    ForwardingProtocol,
    GeoFilterActions,
    HeaderAction,
    HealthProbeRequestType,
    HostNameOperator,
    HttpVersionOperator,
    HttpsRedirect,
    IsDeviceMatchConditionParametersMatchValuesItem,
    IsDeviceOperator,
    KeyVaultSigningKeyParametersTypeName,
    LinkToDefaultDomain,
    LogMetric,
    LogMetricsGranularity,
    LogMetricsGroupBy,
    LogRanking,
    LogRankingMetric,
    ManagedRuleEnabledState,
    ManagedServiceIdentityType,
    MatchProcessingBehavior,
    MatchVariable,
    MetricsGranularity,
    MetricsSeriesUnit,
    MinimumTlsVersion,
    Operator,
    OptimizationType,
    OriginAuthenticationType,
    OriginGroupProvisioningState,
    OriginGroupResourceState,
    OriginProvisioningState,
    OriginResourceState,
    ParamIndicator,
    PolicyEnabledState,
    PolicyMode,
    PolicyResourceState,
    PolicySettingsDefaultCustomBlockResponseStatusCode,
    PostArgsOperator,
    PrivateEndpointStatus,
    ProbeProtocol,
    ProfileProvisioningState,
    ProfileResourceState,
    ProfileScrubbingState,
    ProtocolType,
    ProvisioningState,
    QueryStringBehavior,
    QueryStringCachingBehavior,
    QueryStringOperator,
    RedirectType,
    RemoteAddressOperator,
    RequestBodyOperator,
    RequestHeaderOperator,
    RequestMethodMatchConditionParametersMatchValuesItem,
    RequestMethodOperator,
    RequestSchemeMatchConditionParametersMatchValuesItem,
    RequestSchemeMatchConditionParametersOperator,
    RequestUriOperator,
    ResourceType,
    ResourceUsageUnit,
    ResponseBasedDetectedErrorTypes,
    RuleCacheBehavior,
    RuleIsCompressionEnabled,
    RuleQueryStringCachingBehavior,
    ScrubbingRuleEntryMatchOperator,
    ScrubbingRuleEntryMatchVariable,
    ScrubbingRuleEntryState,
    SecretType,
    SecurityPolicyType,
    ServerPortOperator,
    SharedPrivateLinkResourceStatus,
    SkuName,
    SocketAddrOperator,
    SslProtocol,
    SslProtocolOperator,
    Status,
    Transform,
    TransformType,
    UpdateRule,
    UrlFileExtensionOperator,
    UrlFileNameOperator,
    UrlPathOperator,
    UsageUnit,
    WafAction,
    WafGranularity,
    WafMatchVariable,
    WafMetric,
    WafMetricsGranularity,
    WafMetricsSeriesUnit,
    WafRankingGroupBy,
    WafRankingType,
    WafRuleType,
)
from ._patch import __all__ as _patch_all
from ._patch import *
from ._patch import patch_sdk as _patch_sdk

__all__ = [
    "AFDDomain",
    "AFDDomainHttpsCustomizedCipherSuiteSet",
    "AFDDomainHttpsParameters",
    "AFDDomainProperties",
    "AFDDomainUpdateParameters",
    "AFDDomainUpdatePropertiesParameters",
    "AFDEndpoint",
    "AFDEndpointProperties",
    "AFDEndpointPropertiesUpdateParameters",
    "AFDEndpointUpdateParameters",
    "AFDOrigin",
    "AFDOriginGroup",
    "AFDOriginGroupProperties",
    "AFDOriginGroupUpdateParameters",
    "AFDOriginGroupUpdatePropertiesParameters",
    "AFDOriginProperties",
    "AFDOriginUpdateParameters",
    "AFDOriginUpdatePropertiesParameters",
    "AFDStateProperties",
    "ActivatedResourceReference",
    "AfdPurgeParameters",
    "AfdRouteCacheConfiguration",
    "AzureFirstPartyManagedCertificateParameters",
    "BatchRuleProperties",
    "CacheConfiguration",
    "CacheExpirationActionParameters",
    "CacheKeyQueryStringActionParameters",
    "CanMigrateParameters",
    "CanMigrateProperties",
    "CanMigrateResult",
    "CdnCertificateSourceParameters",
    "CdnEndpoint",
    "CdnManagedHttpsParameters",
    "CdnMigrationToAfdParameters",
    "CdnWebApplicationFirewallPolicy",
    "CdnWebApplicationFirewallPolicyPatchParameters",
    "CdnWebApplicationFirewallPolicyProperties",
    "CertificateSourceParameters",
    "CheckEndpointNameAvailabilityInput",
    "CheckEndpointNameAvailabilityOutput",
    "CheckHostNameAvailabilityInput",
    "CheckNameAvailabilityInput",
    "CheckNameAvailabilityOutput",
    "CidrIpAddress",
    "ClientPortMatchConditionParameters",
    "Components18OrqelSchemasWafmetricsresponsePropertiesSeriesItemsPropertiesDataItems",
    "Components1Gs0LlpSchemasMetricsresponsePropertiesSeriesItemsPropertiesDataItems",
    "ComponentsKpo1PjSchemasWafrankingsresponsePropertiesDataItemsPropertiesMetricsItems",
    "CompressionSettings",
    "ContinentsResponse",
    "ContinentsResponseContinentsItem",
    "ContinentsResponseCountryOrRegionsItem",
    "CookiesMatchConditionParameters",
    "CustomDomain",
    "CustomDomainHttpsParameters",
    "CustomDomainParameters",
    "CustomDomainProperties",
    "CustomDomainPropertiesParameters",
    "CustomRule",
    "CustomRuleList",
    "CustomerCertificateParameters",
    "DeepCreatedCustomDomain",
    "DeepCreatedCustomDomainProperties",
    "DeepCreatedOrigin",
    "DeepCreatedOriginGroup",
    "DeepCreatedOriginGroupProperties",
    "DeepCreatedOriginProperties",
    "DeliveryRule",
    "DeliveryRuleAction",
    "DeliveryRuleActionParameters",
    "DeliveryRuleCacheExpirationAction",
    "DeliveryRuleCacheKeyQueryStringAction",
    "DeliveryRuleClientPortCondition",
    "DeliveryRuleCondition",
    "DeliveryRuleConditionParameters",
    "DeliveryRuleCookiesCondition",
    "DeliveryRuleHostNameCondition",
    "DeliveryRuleHttpVersionCondition",
    "DeliveryRuleIsDeviceCondition",
    "DeliveryRulePostArgsCondition",
    "DeliveryRuleQueryStringCondition",
    "DeliveryRuleRemoteAddressCondition",
    "DeliveryRuleRequestBodyCondition",
    "DeliveryRuleRequestHeaderAction",
    "DeliveryRuleRequestHeaderCondition",
    "DeliveryRuleRequestMethodCondition",
    "DeliveryRuleRequestSchemeCondition",
    "DeliveryRuleRequestUriCondition",
    "DeliveryRuleResponseHeaderAction",
    "DeliveryRuleRouteConfigurationOverrideAction",
    "DeliveryRuleServerPortCondition",
    "DeliveryRuleSocketAddrCondition",
    "DeliveryRuleSslProtocolCondition",
    "DeliveryRuleUrlFileExtensionCondition",
    "DeliveryRuleUrlFileNameCondition",
    "DeliveryRuleUrlPathCondition",
    "DimensionProperties",
    "DomainValidationProperties",
    "EdgeNode",
    "EdgeNodeProperties",
    "Endpoint",
    "EndpointProperties",
    "EndpointPropertiesUpdateParameters",
    "EndpointPropertiesUpdateParametersDeliveryPolicy",
    "EndpointPropertiesUpdateParametersWebApplicationFirewallPolicyLink",
    "EndpointUpdateParameters",
    "ErrorAdditionalInfo",
    "ErrorDetail",
    "ErrorResponse",
    "GeoFilter",
    "HeaderActionParameters",
    "HealthProbeParameters",
    "HostNameMatchConditionParameters",
    "HttpErrorRangeParameters",
    "HttpVersionMatchConditionParameters",
    "IpAddressGroup",
    "IsDeviceMatchConditionParameters",
    "KeyVaultCertificateSourceParameters",
    "KeyVaultSigningKeyParameters",
    "LoadBalancingSettingsParameters",
    "LoadParameters",
    "LogSpecification",
    "ManagedCertificateParameters",
    "ManagedRuleDefinition",
    "ManagedRuleGroupDefinition",
    "ManagedRuleGroupOverride",
    "ManagedRuleOverride",
    "ManagedRuleSet",
    "ManagedRuleSetDefinition",
    "ManagedRuleSetDefinitionProperties",
    "ManagedRuleSetList",
    "ManagedServiceIdentity",
    "MatchCondition",
    "MetricAvailability",
    "MetricSpecification",
    "MetricsResponse",
    "MetricsResponseSeriesItem",
    "MetricsResponseSeriesPropertiesItemsItem",
    "MigrateResult",
    "MigrateResultProperties",
    "MigrationEndpointMapping",
    "MigrationErrorType",
    "MigrationParameters",
    "MigrationWebApplicationFirewallMapping",
    "Operation",
    "OperationDisplay",
    "OperationProperties",
    "Origin",
    "OriginAuthenticationProperties",
    "OriginGroup",
    "OriginGroupOverride",
    "OriginGroupOverrideAction",
    "OriginGroupOverrideActionParameters",
    "OriginGroupProperties",
    "OriginGroupUpdateParameters",
    "OriginGroupUpdatePropertiesParameters",
    "OriginProperties",
    "OriginUpdateParameters",
    "OriginUpdatePropertiesParameters",
    "PolicySettings",
    "PostArgsMatchConditionParameters",
    "Profile",
    "ProfileChangeSkuWafMapping",
    "ProfileLogScrubbing",
    "ProfileProperties",
    "ProfilePropertiesUpdateParameters",
    "ProfileScrubbingRules",
    "ProfileUpdateParameters",
    "ProfileUpgradeParameters",
    "ProxyResource",
    "PurgeParameters",
    "QueryStringMatchConditionParameters",
    "RankingsResponse",
    "RankingsResponseTablesItem",
    "RankingsResponseTablesPropertiesItemsItem",
    "RankingsResponseTablesPropertiesItemsMetricsItem",
    "RateLimitRule",
    "RateLimitRuleList",
    "RemoteAddressMatchConditionParameters",
    "RequestBodyMatchConditionParameters",
    "RequestHeaderMatchConditionParameters",
    "RequestMethodMatchConditionParameters",
    "RequestSchemeMatchConditionParameters",
    "RequestUriMatchConditionParameters",
    "Resource",
    "ResourceReference",
    "ResourceUsage",
    "ResourcesResponse",
    "ResourcesResponseCustomDomainsItem",
    "ResourcesResponseEndpointsItem",
    "ResourcesResponseEndpointsPropertiesItemsItem",
    "ResponseBasedOriginErrorDetectionParameters",
    "Route",
    "RouteConfigurationOverrideActionParameters",
    "RouteProperties",
    "RouteUpdateParameters",
    "RouteUpdatePropertiesParameters",
    "Rule",
    "RuleProperties",
    "RuleSet",
    "RuleSetProperties",
    "RuleUpdateParameters",
    "RuleUpdatePropertiesParameters",
    "Secret",
    "SecretParameters",
    "SecretProperties",
    "SecurityPolicy",
    "SecurityPolicyProperties",
    "SecurityPolicyPropertiesParameters",
    "SecurityPolicyUpdateParameters",
    "SecurityPolicyUpdateProperties",
    "SecurityPolicyWebApplicationFirewallAssociation",
    "SecurityPolicyWebApplicationFirewallParameters",
    "ServerPortMatchConditionParameters",
    "ServiceSpecification",
    "SharedPrivateLinkResourceProperties",
    "Sku",
    "SocketAddrMatchConditionParameters",
    "SslProtocolMatchConditionParameters",
    "SsoUri",
    "SupportedOptimizationTypesListResult",
    "SystemData",
    "TrackedResource",
    "UrlFileExtensionMatchConditionParameters",
    "UrlFileNameMatchConditionParameters",
    "UrlPathMatchConditionParameters",
    "UrlRedirectAction",
    "UrlRedirectActionParameters",
    "UrlRewriteAction",
    "UrlRewriteActionParameters",
    "UrlSigningAction",
    "UrlSigningActionParameters",
    "UrlSigningKey",
    "UrlSigningKeyParameters",
    "UrlSigningParamIdentifier",
    "Usage",
    "UsageName",
    "UserAssignedIdentity",
    "UserManagedHttpsParameters",
    "ValidateCustomDomainInput",
    "ValidateCustomDomainOutput",
    "ValidateProbeInput",
    "ValidateProbeOutput",
    "ValidateSecretInput",
    "ValidateSecretOutput",
    "WafMetricsResponse",
    "WafMetricsResponseSeriesItem",
    "WafMetricsResponseSeriesPropertiesItemsItem",
    "WafRankingsResponse",
    "WafRankingsResponseDataItem",
    "AFDEndpointProtocols",
    "ActionType",
    "AfdCertificateType",
    "AfdCipherSuiteSetType",
    "AfdCustomizedCipherSuiteForTls12",
    "AfdCustomizedCipherSuiteForTls13",
    "AfdMinimumTlsVersion",
    "AfdProvisioningState",
    "AfdQueryStringCachingBehavior",
    "Algorithm",
    "AutoGeneratedDomainNameLabelScope",
    "CacheBehavior",
    "CacheType",
    "CanMigrateDefaultSku",
    "CertificateSource",
    "CertificateSourceParametersType",
    "CertificateType",
    "ClientPortOperator",
    "CookiesOperator",
    "CreatedByType",
    "CustomDomainResourceState",
    "CustomHttpsProvisioningState",
    "CustomHttpsProvisioningSubstate",
    "CustomRuleEnabledState",
    "DeleteRule",
    "DeliveryRuleActionEnum",
    "DeliveryRuleActionParametersType",
    "DeliveryRuleConditionParametersType",
    "DeploymentStatus",
    "DestinationProtocol",
    "DomainValidationState",
    "EnabledState",
    "EndpointProvisioningState",
    "EndpointResourceState",
    "ForwardingProtocol",
    "GeoFilterActions",
    "HeaderAction",
    "HealthProbeRequestType",
    "HostNameOperator",
    "HttpVersionOperator",
    "HttpsRedirect",
    "IsDeviceMatchConditionParametersMatchValuesItem",
    "IsDeviceOperator",
    "KeyVaultSigningKeyParametersTypeName",
    "LinkToDefaultDomain",
    "LogMetric",
    "LogMetricsGranularity",
    "LogMetricsGroupBy",
    "LogRanking",
    "LogRankingMetric",
    "ManagedRuleEnabledState",
    "ManagedServiceIdentityType",
    "MatchProcessingBehavior",
    "MatchVariable",
    "MetricsGranularity",
    "MetricsSeriesUnit",
    "MinimumTlsVersion",
    "Operator",
    "OptimizationType",
    "OriginAuthenticationType",
    "OriginGroupProvisioningState",
    "OriginGroupResourceState",
    "OriginProvisioningState",
    "OriginResourceState",
    "ParamIndicator",
    "PolicyEnabledState",
    "PolicyMode",
    "PolicyResourceState",
    "PolicySettingsDefaultCustomBlockResponseStatusCode",
    "PostArgsOperator",
    "PrivateEndpointStatus",
    "ProbeProtocol",
    "ProfileProvisioningState",
    "ProfileResourceState",
    "ProfileScrubbingState",
    "ProtocolType",
    "ProvisioningState",
    "QueryStringBehavior",
    "QueryStringCachingBehavior",
    "QueryStringOperator",
    "RedirectType",
    "RemoteAddressOperator",
    "RequestBodyOperator",
    "RequestHeaderOperator",
    "RequestMethodMatchConditionParametersMatchValuesItem",
    "RequestMethodOperator",
    "RequestSchemeMatchConditionParametersMatchValuesItem",
    "RequestSchemeMatchConditionParametersOperator",
    "RequestUriOperator",
    "ResourceType",
    "ResourceUsageUnit",
    "ResponseBasedDetectedErrorTypes",
    "RuleCacheBehavior",
    "RuleIsCompressionEnabled",
    "RuleQueryStringCachingBehavior",
    "ScrubbingRuleEntryMatchOperator",
    "ScrubbingRuleEntryMatchVariable",
    "ScrubbingRuleEntryState",
    "SecretType",
    "SecurityPolicyType",
    "ServerPortOperator",
    "SharedPrivateLinkResourceStatus",
    "SkuName",
    "SocketAddrOperator",
    "SslProtocol",
    "SslProtocolOperator",
    "Status",
    "Transform",
    "TransformType",
    "UpdateRule",
    "UrlFileExtensionOperator",
    "UrlFileNameOperator",
    "UrlPathOperator",
    "UsageUnit",
    "WafAction",
    "WafGranularity",
    "WafMatchVariable",
    "WafMetric",
    "WafMetricsGranularity",
    "WafMetricsSeriesUnit",
    "WafRankingGroupBy",
    "WafRankingType",
    "WafRuleType",
]
__all__.extend([p for p in _patch_all if p not in __all__])  # pyright: ignore
_patch_sdk()



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
import pytest
from azure.mgmt.azurestackhcivm.aio import AzureStackHCIVmClient

from devtools_testutils import AzureMgmtRecordedTestCase, RandomNameResourceGroupPreparer
from devtools_testutils.aio import recorded_by_proxy_async

AZURE_LOCATION = "eastus"


@pytest.mark.skip("you may need to update the auto-generated test case before run it")
class TestAzureStackHCIVmLogicalNetworksOperationsAsync(AzureMgmtRecordedTestCase):
    def setup_method(self, method):
        self.client = self.create_mgmt_client(AzureStackHCIVmClient, is_async=True)

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_logical_networks_get(self, resource_group):
        response = await self.client.logical_networks.get(
            resource_group_name=resource_group.name,
            logical_network_name="str",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_logical_networks_begin_create_or_update(self, resource_group):
        response = await (
            await self.client.logical_networks.begin_create_or_update(
                resource_group_name=resource_group.name,
                logical_network_name="str",
                resource={
                    "location": "str",
                    "extendedLocation": {"name": "str", "type": "str"},
                    "id": "str",
                    "name": "str",
                    "properties": {
                        "dhcpOptions": {"dnsServers": ["str"]},
                        "networkType": "str",
                        "provisioningState": "str",
                        "status": {
                            "errorCode": "str",
                            "errorMessage": "str",
                            "provisioningStatus": {"operationId": "str", "status": "str"},
                        },
                        "subnets": [
                            {
                                "name": "str",
                                "properties": {
                                    "addressPrefix": "str",
                                    "addressPrefixes": ["str"],
                                    "ipAllocationMethod": "str",
                                    "ipConfigurationReferences": [{"ID": "str"}],
                                    "ipPools": [
                                        {
                                            "end": "str",
                                            "info": {"available": "str", "used": "str"},
                                            "ipPoolType": "str",
                                            "name": "str",
                                            "start": "str",
                                        }
                                    ],
                                    "networkSecurityGroup": {"id": "str"},
                                    "routeTable": {
                                        "etag": "str",
                                        "name": "str",
                                        "properties": {
                                            "routes": [
                                                {
                                                    "name": "str",
                                                    "properties": {"addressPrefix": "str", "nextHopIpAddress": "str"},
                                                }
                                            ]
                                        },
                                        "type": "str",
                                    },
                                    "vlan": 0,
                                },
                            }
                        ],
                        "vmSwitchName": "str",
                    },
                    "systemData": {
                        "createdAt": "2020-02-20 00:00:00",
                        "createdBy": "str",
                        "createdByType": "str",
                        "lastModifiedAt": "2020-02-20 00:00:00",
                        "lastModifiedBy": "str",
                        "lastModifiedByType": "str",
                    },
                    "tags": {"str": "str"},
                    "type": "str",
                },
            )
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_logical_networks_begin_update(self, resource_group):
        response = await (
            await self.client.logical_networks.begin_update(
                resource_group_name=resource_group.name,
                logical_network_name="str",
                properties={"tags": {"str": "str"}},
            )
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_logical_networks_begin_delete(self, resource_group):
        response = await (
            await self.client.logical_networks.begin_delete(
                resource_group_name=resource_group.name,
                logical_network_name="str",
            )
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_logical_networks_list_by_resource_group(self, resource_group):
        response = self.client.logical_networks.list_by_resource_group(
            resource_group_name=resource_group.name,
        )
        result = [r async for r in response]
        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_logical_networks_list_all(self, resource_group):
        response = self.client.logical_networks.list_all()
        result = [r async for r in response]
        # please add some check logic here by yourself
        # ...



# Copyright (c) Microsoft Corporation.
# Licensed under the MIT license.
"""Sample 15 — Annotations — Attaching file references, citations, and URLs.

Messages can carry annotations that reference files, cite sources, or link
to URLs.  This sample shows how to emit ``file_path``, ``file_citation``,
and ``url_citation`` annotations using the convenience
``output_item_message(text, annotations=[...])`` API.

Usage::

    python sample_15_annotations.py

    curl -X POST http://localhost:8088/responses \
        -H "Content-Type: application/json" \
        -d '{"model": "annotated", "input": "Show me the sources"}'
"""

from azure.ai.agentserver.responses import (
    CreateResponse,
    ResponseContext,
    ResponseEventStream,
    ResponsesAgentServerHost,
)
from azure.ai.agentserver.responses.models import (
    FileCitationBody,
    FilePath,
    UrlCitationBody,
)

app = ResponsesAgentServerHost()


@app.create("annotations")
async def annotations_handler(request: CreateResponse, context: ResponseContext):
    """Return a message with file_path, file_citation, and url_citation annotations."""
    stream = ResponseEventStream(response_id=context.response_id, request=request)
    yield stream.emit_created()
    yield stream.emit_in_progress()

    annotations = [
        FilePath(file_id="/reports/monthly-summary.pdf", index=0),
        FilePath(file_id="/exports/data.csv", index=1),
        FileCitationBody(file_id="/sources/research-paper.pdf", index=2, filename="research-paper.pdf"),
        UrlCitationBody(
            url="https://example.com/docs/guide",
            start_index=0,
            end_index=29,
            title="Developer Guide",
        ),
    ]

    async for event in stream.aoutput_item_message(
        "Here are your files and sources.",
        annotations=annotations,
    ):
        yield event

    yield stream.emit_completed()


if __name__ == "__main__":
    import uvicorn

    uvicorn.run(app.build(), host="0.0.0.0", port=8088)
